<?php
/**
 * @version		1.5.3.47 models/websites.php
 * @package		J2XML
 * @subpackage	com_j2xml
 * @since		1.5.3
 *
 * @author		Helios Ciancio <info@eshiol.it>
 * @link		http://www.eshiol.it
 * @copyright	Copyright (C) 2010 Helios Ciancio. All Rights Reserved
 * @license		http://www.gnu.org/licenses/gpl-3.0.html GNU/GPL v3
 * J2XML is free software. This version may have been modified pursuant
 * to the GNU General Public License, and as distributed it includes or
 * is derivative of works licensed under the GNU General Public License or
 * other free or open source software licenses.
 */
 
// no direct access
defined('_JEXEC') or die('Restricted access.');

jimport( 'joomla.application.component.model' );

class J2XMLModelWebsites extends JModel
{
    var $_data			= null;	
	var $_total			= null;
	var $_pagination	= null;
	var $_context		= 'com_j2xml.website.list.';
	var $_prefix		= 'com_j2xml_website_list_';
	var	$_app;
	
	function __construct()
	{
		parent::__construct();
		
		$this->_app =& JFactory::getApplication();
	
		$app = $this->_app;
		$context = $this->_context;
		$prefix = $this->_prefix;
		
		// Get the pagination request variables
		$limit		= $app->getUserStateFromRequest('global.list.limit',	'limit',				$app->getCfg('list_limit'),	'int');
		$limitstart	= $app->getUserStateFromRequest($context.'limitstart',	$prefix.'limitstart',	0,							'int');

		// In case limit has been changed, adjust limitstart accordingly
		$limitstart = ($limit != 0 ? (floor($limitstart / $limit) * $limit) : 0);

		$this->setState('limit', $limit);
		$this->setState('limitstart', $limitstart);
	}
	
	function _buildQuery()
	{
        $query = 'SELECT a.*'
				.' FROM #__j2xml_websites AS a'
				.$this->_buildWhere()
				.$this->_buildOrderBy()
				;
				
		return $query;
	}

	function getData()
	{		
		$prefix = $this->_prefix;

		if (empty($this->_data))
		{
			$query = $this->_buildQuery();
			$this->_data = $this->_getList($query, $this->getState($prefix.'limitstart'), $this->getState('limit'));
		}
		return $this->_data;	
	}
	
	function _buildOrderBy()
	{
		$app = $this->_app;
//		$context = $this->_context;
//		$prefix = $this->_prefix;
		
//		$filter_order		= $app->getUserStateFromRequest($context.'filter_order',		$prefix.'filter_order',		'id',	'cmd');
		$filter_order		= $app->getUserStateFromRequest('filter_order',		'filter_order',		'id',	'cmd');
//		$filter_order_Dir	= $app->getUserStateFromRequest($context.'filter_order_Dir',	$prefix.'filter_order_Dir',	'',		'word');
		$filter_order_Dir	= $app->getUserStateFromRequest('filter_order_Dir',	'filter_order_Dir',	'',		'word');

		$orderby = ' ORDER BY '.$filter_order.' '.$filter_order_Dir;

		return $orderby;
	}

	function _buildWhere()
	{
		$app = $this->_app;
		$context = $this->_context;
		$prefix = $this->_prefix;

		$filter_state	= $app->getUserStateFromRequest($context.'filter_state',	$prefix.'filter_state',	'',	'word');
	    $search			= $app->getUserStateFromRequest($context.'search', 			$prefix.'search',		'',	'string');
		$search			= JString::strtolower($search);

		$where = array();   
		
		/*
		 * Add the filter specific information to the where clause
		 */
		// Keyword filter
		if ($search) {
			$where[] = '(LOWER(a.title) LIKE '.$this->_db->Quote('%'.$this->_db->getEscaped($search, true).'%', false)
				.' OR a.id = '.(int)$search.')';
		}
		
		// State filter
 		if (JRequest::getCmd('task') == 'website.element') {
			$where[] = 'a.published = 1';
		} else if ($filter_state) {
			if ($filter_state == 'P') {
				$where[] = 'a.published = 1';
			} else if ($filter_state == 'U') {
				$where[] = 'a.published = 0';
			}
		}
		
		// Build the where clause of the record query
		$where = (count($where) ? ' WHERE '.implode(' AND ', $where) : '');	    

	    return $where;
	}
	
	function getTotal()
	{
		// Lets load the content if it doesn't already exist
		if (empty($this->_total))
		{
			$query = $this->_buildQuery();
			$this->_total = $this->_getListCount($query);
		}

		return $this->_total;
	}
	
	function getPagination()
	{
		$prefix = $this->_prefix;
		
		// Lets load the content if it doesn't already exist
		if (empty($this->_pagination))
		{
			jimport('joomla.html.pagination');
			$this->_pagination = new JPagination(
				$this->getTotal(), 
				$this->getState($prefix.'limitstart'), 
				$this->getState('limit') 
				);
		}

		return $this->_pagination;
	}
}
?>